<section itemscope itemtype="http://schema.org/{{ type }}">
    <meta itemprop="@context" content="http://schema.org/extensions" />
    <meta itemprop="@type" content="{{ type }}" />
    <div itemprop="{{ type }}" style="mso-hide:all;display:none;max-height:0px;overflow:hidden;">{{ payload }}</div>
</section>